@@ -1,9 +1,12 @@
 use crate::config::Config;
 use crate::config::Mode;
+use crate::explorer;
 use crate::input::Key;
+use crate::lua;
 use crate::ui::Layout;
 use anyhow::{bail, Result};
 use chrono::{DateTime, Local};
+use humansize::{file_size_opts as options, FileSize};
 use indexmap::set::IndexSet;
 use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
@@ -11,17 +14,21 @@ use std::collections::HashMap;
 use std::collections::VecDeque;
 use std::env;
 use std::fs;
-use std::io;
 use std::path::PathBuf;
 
+pub const VERSION: &str = env!("CARGO_PKG_VERSION");
 pub const TEMPLATE_TABLE_ROW: &str = "TEMPLATE_TABLE_ROW";
 pub const UNSUPPORTED_STR: &str = "???";
-pub const UPGRADE_GUIDE_LINK: &str = "https://github.com/sayanarijit/xplr/wiki/Upgrade-Guide";
+
+fn to_humansize(size: u64) -> String {
+    size.file_size(options::CONVENTIONAL)
+        .unwrap_or_else(|_| format!("{} B", size))
+}
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Pipe {
+    path: String,
     msg_in: String,
-    focus_out: String,
     selection_out: String,
     result_out: String,
     directory_nodes_out: String,
@@ -32,44 +39,31 @@ pub struct Pipe {
 
 impl Pipe {
     fn from_session_path(path: &str) -> Result<Self> {
-        let pipesdir = PathBuf::from(path).join("pipe");
+        let path = PathBuf::from(path).join("pipe");
 
-        fs::create_dir_all(&pipesdir).unwrap();
+        let msg_in = path.join("msg_in").to_string_lossy().to_string();
 
-        let msg_in = pipesdir.join("msg_in").to_string_lossy().to_string();
+        let selection_out = path.join("selection_out").to_string_lossy().to_string();
 
-        let focus_out = pipesdir.join("focus_out").to_string_lossy().to_string();
+        let result_out = path.join("result_out").to_string_lossy().to_string();
 
-        let selection_out = pipesdir.join("selection_out").to_string_lossy().to_string();
-
-        let result_out = pipesdir.join("result_out").to_string_lossy().to_string();
-
-        let directory_nodes_out = pipesdir
+        let directory_nodes_out = path
             .join("directory_nodes_out")
             .to_string_lossy()
             .to_string();
 
-        let global_help_menu_out = pipesdir
+        let global_help_menu_out = path
             .join("global_help_menu_out")
             .to_string_lossy()
             .to_string();
 
-        let logs_out = pipesdir.join("logs_out").to_string_lossy().to_string();
+        let logs_out = path.join("logs_out").to_string_lossy().to_string();
 
-        let history_out = pipesdir.join("history_out").to_string_lossy().to_string();
-
-        fs::write(&msg_in, "")?;
-        fs::write(&focus_out, "")?;
-        fs::write(&selection_out, "")?;
-        fs::write(&directory_nodes_out, "")?;
-        fs::write(&global_help_menu_out, "")?;
-        fs::write(&result_out, "")?;
-        fs::write(&logs_out, "")?;
-        fs::write(&history_out, "")?;
+        let history_out = path.join("history_out").to_string_lossy().to_string();
 
         Ok(Self {
+            path: path.to_string_lossy().to_string(),
             msg_in,
-            focus_out,
             selection_out,
             result_out,
             directory_nodes_out,
@@ -113,6 +107,11 @@ impl Pipe {
     pub fn history_out(&self) -> &String {
         &self.history_out
     }
+
+    /// Get a reference to the pipe's path.
+    pub fn path(&self) -> &String {
+        &self.path
+    }
 }
 
 #[derive(Debug, Clone, Eq, Hash, PartialEq, Serialize, Deserialize)]
@@ -124,6 +123,7 @@ pub struct ResolvedNode {
     is_readonly: bool,
     mime_essence: String,
     size: u64,
+    human_size: String,
 }
 
 impl ResolvedNode {
@@ -143,6 +143,8 @@ impl ResolvedNode {
             .map(|m| m.essence_str().to_string())
             .unwrap_or_default();
 
+        let human_size = to_humansize(size);
+
         Self {
             absolute_path: path.to_string_lossy().to_string(),
             extension,
@@ -151,6 +153,7 @@ impl ResolvedNode {
             is_readonly,
             mime_essence,
             size,
+            human_size,
         }
     }
 
@@ -188,6 +191,11 @@ impl ResolvedNode {
     pub fn size(&self) -> u64 {
         self.size
     }
+
+    /// Get a reference to the resolved node's human size.
+    pub fn human_size(&self) -> &String {
+        &self.human_size
+    }
 }
 
 #[derive(Debug, Clone, Eq, Hash, PartialEq, Serialize, Deserialize)]
@@ -203,6 +211,7 @@ pub struct Node {
     is_readonly: bool,
     mime_essence: String,
     size: u64,
+    human_size: String,
     canonical: Option<ResolvedNode>,
     symlink: Option<ResolvedNode>,
 }
@@ -244,6 +253,8 @@ impl Node {
             .map(|m| m.essence_str().to_string())
             .unwrap_or_default();
 
+        let human_size = to_humansize(size);
+
         Self {
             parent,
             relative_path,
@@ -256,6 +267,7 @@ impl Node {
             is_readonly,
             mime_essence,
             size,
+            human_size,
             canonical: maybe_canonical_meta.clone(),
             symlink: if is_symlink {
                 maybe_canonical_meta
@@ -329,6 +341,11 @@ impl Node {
     pub fn absolute_path(&self) -> &String {
         &self.absolute_path
     }
+
+    /// Get a reference to the node's human size.
+    pub fn human_size(&self) -> &String {
+        &self.human_size
+    }
 }
 
 impl Ord for Node {
@@ -1113,6 +1130,17 @@ pub enum ExternalMsg {
     /// **Example:** `CallSilently: {command: tput, args: ["bell"]}`
     CallSilently(Command),
 
+    /// Call a Lua function
+    ///
+    /// **Example:** `CallLua: custom.foo_funtion`
+    CallLua(String),
+
+    /// Like `CallLua` but without the flicker. The stdin, stdout
+    /// stderr will be piped to null. So it's non-interactive.
+    ///
+    /// **Example:** `CallLuaSilently: custom.bar_function`
+    CallLuaSilently(String),
+
     /// An alias to `Call: {command: bash, args: ["-c", "${command}"], silent: false}`
     /// where ${command} is the given value.
     ///
@@ -1302,7 +1330,6 @@ impl Command {
 
 #[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub enum MsgOut {
-    ExplorePwd,
     ExplorePwdAsync,
     ExploreParentsAsync,
     Refresh,
@@ -1313,6 +1340,8 @@ pub enum MsgOut {
     Debug(String),
     Call(Command),
     CallSilently(Command),
+    CallLua(String),
+    CallLuaSilently(String),
     Enque(Task),
 }
 
@@ -1395,9 +1424,11 @@ pub struct History {
 
 impl History {
     fn push(mut self, path: String) -> Self {
-        self.paths = self.paths.into_iter().take(self.loc + 1).collect();
-        self.paths.push(path);
-        self.loc = self.paths.len().max(1) - 1;
+        if self.peek() != Some(&path) {
+            self.paths = self.paths.into_iter().take(self.loc + 1).collect();
+            self.paths.push(path);
+            self.loc = self.paths.len().max(1) - 1;
+        }
         self
     }
 
@@ -1438,34 +1469,22 @@ pub struct App {
 }
 
 impl App {
-    pub fn create(pwd: PathBuf) -> Result<Self> {
+    pub fn create(pwd: PathBuf, lua: &mlua::Lua) -> Result<Self> {
+        let config = lua::init(lua)?;
+
         let config_dir = dirs::config_dir()
             .unwrap_or_else(|| PathBuf::from("."))
             .join("xplr");
 
-        let config_file = config_dir.join("config.yml");
-        let default_config = Config::default();
-        let default_config_version = default_config.version().clone();
+        let lua_script_file = config_dir.join("init.lua");
 
-        let config: Config = if config_file.exists() {
-            let c: Config =
-                serde_yaml::from_reader(io::BufReader::new(&fs::File::open(&config_file)?))?;
-            c.extended()
+        let (config, load_err) = if lua_script_file.exists() {
+            match lua::extend(lua, &lua_script_file.to_string_lossy().to_string()) {
+                Ok(c) => (c, None),
+                Err(e) => (config, Some(e.to_string())),
+            }
         } else {
-            default_config
-        };
-
-        if !config.is_compatible()? {
-            bail!(
-                "incompatible configuration version in {}
-                You config version is : {}
-                Required version is   : {}
-                Visit {}",
-                config_file.to_string_lossy().to_string(),
-                config.version(),
-                default_config_version,
-                UPGRADE_GUIDE_LINK,
-            )
+            (config, None)
         };
 
         let mode = match config.modes().get(
@@ -1517,12 +1536,13 @@ impl App {
             explorer_config.sorters = sorters.clone();
         };
 
+        env::set_current_dir(&pwd)?;
         let pwd = pwd.to_string_lossy().to_string();
 
-        let mut app = Self {
-            version: Config::default().version().clone(),
-            config: config.clone(),
-            pwd: pwd.clone(),
+        let app = Self {
+            version: VERSION.to_string(),
+            config,
+            pwd,
             directory_buffers: Default::default(),
             selection: Default::default(),
             msg_out: Default::default(),
@@ -1537,24 +1557,13 @@ impl App {
             logs_hidden: Default::default(),
             history: Default::default(),
             last_modes: Default::default(),
-        }
-        .change_directory(&pwd)?;
+        };
 
-        if let Some(notif) = config.upgrade_notification()? {
-            let notif = format!(
-                "{}. To stop seeing this log, update your config version from {} to {}.",
-                &notif,
-                config.version(),
-                app.version()
-            );
-            app = app.enqueue(Task::new(
-                MsgIn::External(ExternalMsg::LogInfo(notif)),
-                None,
-            ));
+        if let Some(err) = load_err {
+            app.log_error(err)
+        } else {
+            Ok(app)
         }
-
-        app.write_pipes()?;
-        Ok(app)
     }
 
     pub fn focused_node(&self) -> Option<&Node> {
@@ -1596,7 +1605,7 @@ impl App {
                 ExternalMsg::ExplorePwdAsync => self.explore_pwd_async(),
                 ExternalMsg::Refresh => self.refresh(),
                 ExternalMsg::ClearScreen => self.clear_screen(),
-                ExternalMsg::FocusFirst => self.focus_first(),
+                ExternalMsg::FocusFirst => self.focus_first(true),
                 ExternalMsg::FocusLast => self.focus_last(),
                 ExternalMsg::FocusPrevious => self.focus_previous(),
                 ExternalMsg::FocusPreviousByRelativeIndex(i) => {
@@ -1611,12 +1620,12 @@ impl App {
                 ExternalMsg::FocusNextByRelativeIndexFromInput => {
                     self.focus_next_by_relative_index_from_input()
                 }
-                ExternalMsg::FocusPath(p) => self.focus_path(&p),
+                ExternalMsg::FocusPath(p) => self.focus_path(&p, true),
                 ExternalMsg::FocusPathFromInput => self.focus_path_from_input(),
                 ExternalMsg::FocusByIndex(i) => self.focus_by_index(i),
                 ExternalMsg::FocusByIndexFromInput => self.focus_by_index_from_input(),
-                ExternalMsg::FocusByFileName(n) => self.focus_by_file_name(&n),
-                ExternalMsg::ChangeDirectory(dir) => self.change_directory(&dir),
+                ExternalMsg::FocusByFileName(n) => self.focus_by_file_name(&n, true),
+                ExternalMsg::ChangeDirectory(dir) => self.change_directory(&dir, true),
                 ExternalMsg::Enter => self.enter(),
                 ExternalMsg::Back => self.back(),
                 ExternalMsg::LastVisitedPath => self.last_visited_path(),
@@ -1639,6 +1648,8 @@ impl App {
                 ExternalMsg::SwitchLayoutCustom(mode) => self.switch_layout_custom(&mode),
                 ExternalMsg::Call(cmd) => self.call(cmd),
                 ExternalMsg::CallSilently(cmd) => self.call_silently(cmd),
+                ExternalMsg::CallLua(func) => self.call_lua(func),
+                ExternalMsg::CallLuaSilently(func) => self.call_lua_silently(func),
                 ExternalMsg::BashExec(cmd) => self.bash_exec(cmd),
                 ExternalMsg::BashExecSilently(cmd) => self.bash_exec_silently(cmd),
                 ExternalMsg::Select => self.select(),
@@ -1721,9 +1732,13 @@ impl App {
         Ok(self)
     }
 
-    fn explore_pwd(mut self) -> Result<Self> {
-        self.msg_out.push_back(MsgOut::ExplorePwd);
-        Ok(self)
+    pub fn explore_pwd(self) -> Result<Self> {
+        let dir = explorer::explore_sync(
+            self.explorer_config().clone(),
+            self.pwd().clone(),
+            self.focused_node().map(|n| n.absolute_path().clone()),
+        )?;
+        self.add_directory(dir.parent().clone(), dir)
     }
 
     fn explore_pwd_async(mut self) -> Result<Self> {
@@ -1746,36 +1761,70 @@ impl App {
         Ok(self)
     }
 
-    fn focus_first(mut self) -> Result<Self> {
+    pub fn focus_first(mut self, save_history: bool) -> Result<Self> {
+        let mut history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
+            if save_history {
+                if let Some(n) = dir.focused_node() {
+                    history = history.push(n.absolute_path().clone());
+                }
+            }
+
             dir.focus = 0;
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+
+            if save_history {
+                if let Some(n) = self.clone().focused_node() {
+                    self.history = history.push(n.absolute_path().clone())
+                }
+            }
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_last(mut self) -> Result<Self> {
+        let mut history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
+            if let Some(n) = dir.focused_node() {
+                history = history.push(n.absolute_path().clone());
+            }
+
             dir.focus = dir.total.max(1) - 1;
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+
+            if let Some(n) = dir.focused_node() {
+                self.history = history.push(n.absolute_path().clone());
+            }
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_previous(mut self) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
             dir.focus = dir.focus.max(1) - 1;
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_previous_by_relative_index(mut self, index: usize) -> Result<Self> {
+        let mut history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
+            if let Some(n) = dir.focused_node() {
+                history = history.push(n.absolute_path().clone());
+            }
+
             dir.focus = dir.focus.max(index) - index;
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+            if let Some(n) = self.focused_node() {
+                self.history = history.push(n.absolute_path().clone());
+            }
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_previous_by_relative_index_from_input(self) -> Result<Self> {
@@ -1789,17 +1838,27 @@ impl App {
     fn focus_next(mut self) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
             dir.focus = (dir.focus + 1).min(dir.total.max(1) - 1);
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_next_by_relative_index(mut self, index: usize) -> Result<Self> {
+        let mut history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
+            if let Some(n) = dir.focused_node() {
+                history = history.push(n.absolute_path().clone());
+            }
+
             dir.focus = (dir.focus + index).min(dir.total.max(1) - 1);
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+            if let Some(n) = self.focused_node() {
+                self.history = history.push(n.absolute_path().clone());
+            }
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_next_by_relative_index_from_input(self) -> Result<Self> {
@@ -1815,17 +1874,19 @@ impl App {
             .focused_node()
             .and_then(|n| n.symlink.to_owned().map(|s| s.absolute_path))
         {
-            self.focus_path(&pth)
+            self.focus_path(&pth, true)
         } else {
             Ok(self)
         }
     }
 
-    fn change_directory(mut self, dir: &str) -> Result<Self> {
+    fn change_directory(mut self, dir: &str, save_history: bool) -> Result<Self> {
         match env::set_current_dir(dir) {
             Ok(()) => {
                 self.pwd = dir.to_owned();
-                self.history = self.history.push(self.pwd.clone());
+                if save_history {
+                    self.history = self.history.push(format!("{}/", self.pwd));
+                }
                 self.explore_pwd_async()
             }
             Err(e) => self.log_error(e.to_string()),
@@ -1835,7 +1896,7 @@ impl App {
     fn enter(self) -> Result<Self> {
         self.focused_node()
             .map(|n| n.absolute_path.clone())
-            .map(|p| self.clone().change_directory(&p))
+            .map(|p| self.clone().change_directory(&p, true))
             .unwrap_or(Ok(self))
     }
 
@@ -1844,31 +1905,41 @@ impl App {
             .parent()
             .map(|p| {
                 self.clone()
-                    .change_directory(&p.to_string_lossy().to_string())
+                    .change_directory(&p.to_string_lossy().to_string(), true)
             })
             .unwrap_or(Ok(self))
     }
 
     fn last_visited_path(mut self) -> Result<Self> {
         self.history = self.history.visit_last();
-        self.pwd = self
-            .history
-            .peek()
-            .map(|p| p.to_owned())
-            .unwrap_or(self.pwd);
-        env::set_current_dir(&self.pwd)?;
-        self.explore_pwd_async()
+        if let Some(target) = self.history.peek() {
+            if target.ends_with('/') {
+                target
+                    .strip_suffix('/')
+                    .map(|s| self.clone().change_directory(s, false))
+                    .unwrap_or(Ok(self))
+            } else {
+                self.clone().focus_path(target, false)
+            }
+        } else {
+            Ok(self)
+        }
     }
 
     fn next_visited_path(mut self) -> Result<Self> {
         self.history = self.history.visit_next();
-        self.pwd = self
-            .history
-            .peek()
-            .map(|p| p.to_owned())
-            .unwrap_or(self.pwd);
-        env::set_current_dir(&self.pwd)?;
-        self.explore_pwd_async()
+        if let Some(target) = self.history.peek() {
+            if target.ends_with('/') {
+                target
+                    .strip_suffix('/')
+                    .map(|s| self.clone().change_directory(s, false))
+                    .unwrap_or(Ok(self))
+            } else {
+                self.clone().focus_path(target, false)?.refresh()
+            }
+        } else {
+            Ok(self)
+        }
     }
 
     fn buffer_input(mut self, input: &str) -> Result<Self> {
@@ -1933,8 +2004,12 @@ impl App {
     }
 
     fn focus_by_index(mut self, index: usize) -> Result<Self> {
+        let history = self.history.clone();
         if let Some(dir) = self.directory_buffer_mut() {
             dir.focus = index.min(dir.total.max(1) - 1);
+            if let Some(n) = self.focused_node() {
+                self.history = history.push(n.absolute_path().clone());
+            }
             self.refresh()
         } else {
             Ok(self)
@@ -1949,7 +2024,8 @@ impl App {
         }
     }
 
-    fn focus_by_file_name(mut self, name: &str) -> Result<Self> {
+    pub fn focus_by_file_name(mut self, name: &str, save_history: bool) -> Result<Self> {
+        let mut history = self.history.clone();
         if let Some(dir_buf) = self.directory_buffer_mut() {
             if let Some(focus) = dir_buf
                 .clone()
@@ -1959,7 +2035,17 @@ impl App {
                 .find(|(_, n)| n.relative_path == name)
                 .map(|(i, _)| i)
             {
+                if save_history {
+                    if let Some(n) = dir_buf.focused_node() {
+                        history = history.push(n.absolute_path().clone());
+                    }
+                }
                 dir_buf.focus = focus;
+                if save_history {
+                    if let Some(n) = dir_buf.focused_node() {
+                        self.history = history.push(n.absolute_path().clone());
+                    }
+                }
                 self.refresh()
             } else {
                 self.log_error(format!("{} not found in $PWD", name))
@@ -1969,12 +2055,12 @@ impl App {
         }
     }
 
-    fn focus_path(self, path: &str) -> Result<Self> {
+    pub fn focus_path(self, path: &str, save_history: bool) -> Result<Self> {
         let pathbuf = PathBuf::from(path);
         if let Some(parent) = pathbuf.parent() {
             if let Some(filename) = pathbuf.file_name() {
-                self.change_directory(&parent.to_string_lossy().to_string())?
-                    .focus_by_file_name(&filename.to_string_lossy().to_string())
+                self.change_directory(&parent.to_string_lossy().to_string(), false)?
+                    .focus_by_file_name(&filename.to_string_lossy().to_string(), save_history)
             } else {
                 self.log_error(format!("{} not found", path))
             }
@@ -1985,7 +2071,7 @@ impl App {
 
     fn focus_path_from_input(self) -> Result<Self> {
         if let Some(p) = self.input_buffer() {
-            self.focus_path(&p)
+            self.focus_path(&p, true)
         } else {
             Ok(self)
         }
@@ -2090,6 +2176,18 @@ impl App {
         Ok(self)
     }
 
+    fn call_lua(mut self, func: String) -> Result<Self> {
+        self.logs_hidden = true;
+        self.msg_out.push_back(MsgOut::CallLua(func));
+        Ok(self)
+    }
+
+    fn call_lua_silently(mut self, func: String) -> Result<Self> {
+        self.logs_hidden = true;
+        self.msg_out.push_back(MsgOut::CallLuaSilently(func));
+        Ok(self)
+    }
+
     fn bash_exec(self, script: String) -> Result<Self> {
         self.call(Command {
             command: "bash".into(),
@@ -2106,16 +2204,16 @@ impl App {
 
     pub fn add_directory(mut self, parent: String, dir: DirectoryBuffer) -> Result<Self> {
         self.directory_buffers.insert(parent, dir);
-        self.msg_out.push_back(MsgOut::Refresh);
-        Ok(self)
+        self.refresh()
     }
 
     fn select(mut self) -> Result<Self> {
         if let Some(n) = self.focused_node().map(|n| n.to_owned()) {
             self.selection.insert(n);
-            self.msg_out.push_back(MsgOut::Refresh);
+            self.refresh()
+        } else {
+            Ok(self)
         }
-        Ok(self)
     }
 
     fn select_path(mut self, path: String) -> Result<Self> {
@@ -2124,9 +2222,10 @@ impl App {
         let filename = path.file_name().map(|p| p.to_string_lossy().to_string());
         if let (Some(p), Some(n)) = (parent, filename) {
             self.selection.insert(Node::new(p, n));
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn select_all(mut self) -> Result<Self> {
@@ -2563,13 +2662,9 @@ impl App {
     pub fn write_pipes(&self) -> Result<()> {
         // TODO optimize and test
 
-        let focused_node_str = self.focused_node_str();
-        // if last_app
-        //     .map(|a| a.focused_node_str() != focused_node_str)
-        //     .unwrap_or(true)
-        // {
-        fs::write(&self.pipe().focus_out, focused_node_str)?;
-        // };
+        fs::create_dir_all(self.pipe().path())?;
+
+        fs::write(&self.pipe().msg_in, "")?;
 
         let selection_str = self.selection_str();
         // if last_app
@@ -2632,6 +2727,23 @@ impl App {
         Ok(())
     }
 
+    pub fn cleanup_pipes(&self) -> Result<()> {
+        while !fs::read_to_string(self.pipe().msg_in())?.is_empty() {
+            std::thread::sleep(std::time::Duration::from_millis(1));
+        }
+
+        fs::remove_file(self.pipe().msg_in())?;
+        fs::remove_file(self.pipe().selection_out())?;
+        fs::remove_file(self.pipe().result_out())?;
+        fs::remove_file(self.pipe().directory_nodes_out())?;
+        fs::remove_file(self.pipe().global_help_menu_out())?;
+        fs::remove_file(self.pipe().logs_out())?;
+        fs::remove_file(self.pipe().history_out())?;
+
+        fs::remove_dir(self.pipe().path())?;
+        Ok(())
+    }
+
     /// Get a reference to the app's layout.
     pub fn layout(&self) -> &Layout {
         &self.layout
