@@ -1,11 +1,12 @@
 use crate::app;
 use crate::app::HelpMenuLine;
 use crate::app::{Node, ResolvedNode};
-use crate::config::BlockConfig;
-use crate::config::Layout;
-use handlebars::Handlebars;
+use crate::config::PanelUiConfig;
+use crate::lua;
 use indexmap::IndexSet;
 use lazy_static::lazy_static;
+use mlua::Lua;
+use mlua::LuaSerdeExt;
 use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
 use std::collections::HashMap;
@@ -23,6 +24,110 @@ lazy_static! {
     pub static ref DEFAULT_STYLE: TuiStyle = TuiStyle::default();
 }
 
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct LayoutOptions {
+    #[serde(default)]
+    margin: Option<u16>,
+    #[serde(default)]
+    horizontal_margin: Option<u16>,
+    #[serde(default)]
+    vertical_margin: Option<u16>,
+    #[serde(default)]
+    constraints: Option<Vec<Constraint>>,
+}
+
+impl LayoutOptions {
+    pub fn extend(mut self, other: Self) -> Self {
+        self.margin = other.margin.or(self.margin);
+        self.horizontal_margin = other.horizontal_margin.or(self.horizontal_margin);
+        self.vertical_margin = other.vertical_margin.or(self.vertical_margin);
+        self.constraints = other.constraints.or(self.constraints);
+        self
+    }
+
+    /// Get a reference to the layout options's constraints.
+    pub fn constraints(&self) -> &Option<Vec<Constraint>> {
+        &self.constraints
+    }
+
+    /// Get a reference to the layout options's margin.
+    pub fn margin(&self) -> Option<u16> {
+        self.margin
+    }
+
+    /// Get a reference to the layout options's horizontal margin.
+    pub fn horizontal_margin(&self) -> Option<u16> {
+        self.horizontal_margin
+    }
+
+    /// Get a reference to the layout options's vertical margin.
+    pub fn vertical_margin(&self) -> Option<u16> {
+        self.vertical_margin
+    }
+}
+
+#[derive(Debug, Clone, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub enum Layout {
+    Nothing,
+    Table,
+    InputAndLogs,
+    Selection,
+    HelpMenu,
+    SortAndFilter,
+    Horizontal {
+        config: LayoutOptions,
+        splits: Vec<Layout>,
+    },
+    Vertical {
+        config: LayoutOptions,
+        splits: Vec<Layout>,
+    },
+}
+
+impl Default for Layout {
+    fn default() -> Self {
+        Self::Nothing
+    }
+}
+
+impl Layout {
+    pub fn extend(self, other: Self) -> Self {
+        match (self, other) {
+            (s, Self::Nothing) => s,
+            (
+                Self::Horizontal {
+                    config: sconfig,
+                    splits: _,
+                },
+                Self::Horizontal {
+                    config: oconfig,
+                    splits: osplits,
+                },
+            ) => Self::Horizontal {
+                config: sconfig.extend(oconfig),
+                splits: osplits,
+            },
+
+            (
+                Self::Vertical {
+                    config: sconfig,
+                    splits: _,
+                },
+                Self::Vertical {
+                    config: oconfig,
+                    splits: osplits,
+                },
+            ) => Self::Vertical {
+                config: sconfig.extend(oconfig),
+                splits: osplits,
+            },
+            (_, other) => other,
+        }
+    }
+}
+
 #[derive(Debug, Clone, Copy, Hash, PartialEq, Eq, PartialOrd, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub enum Border {
@@ -114,8 +219,7 @@ impl Into<TuiStyle> for Style {
                         .unwrap_or_default()
                         .into_iter()
                         .map(|m| m.bits())
-                        .reduce(|a, b| (a ^ b))
-                        .unwrap_or_else(|| TuiModifier::empty().bits()),
+                        .fold(0, |a, b| (a ^ b)),
                 ),
 
                 sub_modifier: TuiModifier::from_bits_truncate(
@@ -123,16 +227,68 @@ impl Into<TuiStyle> for Style {
                         .unwrap_or_default()
                         .into_iter()
                         .map(|m| m.bits())
-                        .reduce(|a, b| (a ^ b))
-                        .unwrap_or_else(|| TuiModifier::empty().bits()),
+                        .fold(0, |a, b| (a ^ b)),
                 ),
             }
         }
     }
 }
 
+#[derive(Debug, Clone, Copy, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub enum Constraint {
+    Percentage(u16),
+    Ratio(u32, u32),
+    Length(u16),
+    LengthLessThanScreenHeight(u16),
+    LengthLessThanScreenWidth(u16),
+    LengthLessThanLayoutHeight(u16),
+    LengthLessThanLayoutWidth(u16),
+    Max(u16),
+    MaxLessThanScreenHeight(u16),
+    MaxLessThanScreenWidth(u16),
+    MaxLessThanLayoutHeight(u16),
+    MaxthLessThanLayoutWidth(u16),
+    Min(u16),
+    MinLessThanScreenHeight(u16),
+    MinLessThanScreenWidth(u16),
+    MinLessThanLayoutHeight(u16),
+    MinLessThanLayoutWidth(u16),
+}
+
+impl Constraint {
+    pub fn to_tui(self, screen_size: Rect, layout_size: Rect) -> TuiConstraint {
+        match self {
+            Self::Percentage(n) => TuiConstraint::Percentage(n),
+            Self::Ratio(x, y) => TuiConstraint::Ratio(x, y),
+            Self::Length(n) => TuiConstraint::Length(n),
+            Self::LengthLessThanScreenHeight(n) => {
+                TuiConstraint::Length(screen_size.height.max(n) - n)
+            }
+            Self::LengthLessThanScreenWidth(n) => {
+                TuiConstraint::Length(screen_size.width.max(n) - n)
+            }
+            Self::LengthLessThanLayoutHeight(n) => {
+                TuiConstraint::Length(layout_size.height.max(n) - n)
+            }
+            Self::LengthLessThanLayoutWidth(n) => {
+                TuiConstraint::Length(layout_size.width.max(n) - n)
+            }
+            Self::Max(n) => TuiConstraint::Max(n),
+            Self::MaxLessThanScreenHeight(n) => TuiConstraint::Max(screen_size.height.max(n) - n),
+            Self::MaxLessThanScreenWidth(n) => TuiConstraint::Max(screen_size.width.max(n) - n),
+            Self::MaxLessThanLayoutHeight(n) => TuiConstraint::Max(layout_size.height.max(n) - n),
+            Self::MaxthLessThanLayoutWidth(n) => TuiConstraint::Max(layout_size.width.max(n) - n),
+            Self::Min(n) => TuiConstraint::Min(n),
+            Self::MinLessThanScreenHeight(n) => TuiConstraint::Min(screen_size.height.max(n) - n),
+            Self::MinLessThanScreenWidth(n) => TuiConstraint::Min(screen_size.width.max(n) - n),
+            Self::MinLessThanLayoutHeight(n) => TuiConstraint::Min(layout_size.height.max(n) - n),
+            Self::MinLessThanLayoutWidth(n) => TuiConstraint::Min(layout_size.width.max(n) - n),
+        }
+    }
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
 pub struct ResolvedNodeUiMetadata {
     absolute_path: String,
     extension: String,
@@ -141,6 +297,7 @@ pub struct ResolvedNodeUiMetadata {
     is_readonly: bool,
     mime_essence: String,
     size: u64,
+    human_size: String,
 }
 
 impl From<ResolvedNode> for ResolvedNodeUiMetadata {
@@ -153,12 +310,12 @@ impl From<ResolvedNode> for ResolvedNodeUiMetadata {
             is_readonly: node.is_readonly(),
             mime_essence: node.mime_essence().clone(),
             size: node.size(),
+            human_size: node.human_size().clone(),
         }
     }
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
 struct NodeUiMetadata {
     // From Node
     parent: String,
@@ -172,6 +329,7 @@ struct NodeUiMetadata {
     is_readonly: bool,
     mime_essence: String,
     size: u64,
+    human_size: String,
     canonical: Option<ResolvedNodeUiMetadata>,
     symlink: Option<ResolvedNodeUiMetadata>,
 
@@ -216,6 +374,7 @@ impl NodeUiMetadata {
             is_readonly: node.is_readonly(),
             mime_essence: node.mime_essence().clone(),
             size: node.size(),
+            human_size: node.human_size().clone(),
             canonical: node.canonical().to_owned().map(|s| s.into()),
             symlink: node.symlink().to_owned().map(|s| s.into()),
             index,
@@ -233,7 +392,7 @@ impl NodeUiMetadata {
     }
 }
 
-fn block<'a>(config: BlockConfig, default_title: String) -> Block<'a> {
+fn block<'a>(config: PanelUiConfig, default_title: String) -> Block<'a> {
     Block::default()
         .borders(TuiBorders::from_bits_truncate(
             config
@@ -242,8 +401,7 @@ fn block<'a>(config: BlockConfig, default_title: String) -> Block<'a> {
                 .unwrap_or_default()
                 .iter()
                 .map(|b| b.bits())
-                .reduce(|a, b| (a ^ b))
-                .unwrap_or_else(|| TuiBorders::NONE.bits()),
+                .fold(0, |a, b| (a ^ b)),
         ))
         .title(Span::styled(
             config.title().format().clone().unwrap_or(default_title),
@@ -253,13 +411,17 @@ fn block<'a>(config: BlockConfig, default_title: String) -> Block<'a> {
 }
 
 fn draw_table<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    hb: &Handlebars,
+    lua: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config
+        .default()
+        .clone()
+        .extend(panel_config.table().clone());
     let app_config = app.config().to_owned();
     let header_height = app_config.general().table().header().height().unwrap_or(1);
     let height: usize = (layout_size.height.max(header_height + 2) - (header_height + 2)).into();
@@ -363,12 +525,27 @@ fn draw_table<B: Backend>(
                         node_type.meta().clone(),
                     );
 
-                    let cols = hb
-                        .render(app::TEMPLATE_TABLE_ROW, &meta)
-                        .ok()
-                        .unwrap_or_else(|| app::UNSUPPORTED_STR.into())
-                        .split('\t')
-                        .map(|x| Cell::from(x.to_string()))
+                    let cols = lua
+                        .to_value::<NodeUiMetadata>(&meta)
+                        .map(|v| {
+                            app_config
+                                .general()
+                                .table()
+                                .row()
+                                .cols()
+                                .clone()
+                                .unwrap_or_default()
+                                .iter()
+                                .filter_map(|c| {
+                                    c.format().as_ref().map(|f| {
+                                        lua::call(lua, f, &v).unwrap_or_else(|e| e.to_string())
+                                    })
+                                })
+                                .collect::<Vec<String>>()
+                        })
+                        .unwrap_or_default()
+                        .iter()
+                        .map(|x| Cell::from(x.to_owned()))
                         .collect::<Vec<Cell>>();
 
                     Row::new(cols).style(style.into())
@@ -430,13 +607,17 @@ fn draw_table<B: Backend>(
 }
 
 fn draw_selection<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config
+        .default()
+        .clone()
+        .extend(panel_config.selection().clone());
     let selection: Vec<ListItem> = app
         .selection()
         .iter()
@@ -457,13 +638,17 @@ fn draw_selection<B: Backend>(
 }
 
 fn draw_help_menu<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config
+        .default()
+        .clone()
+        .extend(panel_config.help_menu().clone());
     let help_menu_rows = app
         .mode()
         .help_menu()
@@ -476,7 +661,7 @@ fn draw_help_menu<B: Backend>(
                     .key_bindings()
                     .remaps()
                     .iter()
-                    .filter(|(_, t)| t == &&k)
+                    .filter(|(_, maybeto)| maybeto.as_ref().map(|to| to == &k).unwrap_or(false))
                     .map(|(f, _)| f.clone())
                     .collect::<Vec<String>>()
                     .join("|");
@@ -505,13 +690,17 @@ fn draw_help_menu<B: Backend>(
 }
 
 fn draw_input_buffer<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config
+        .default()
+        .clone()
+        .extend(panel_config.input_and_logs().clone());
     let input_buf = Paragraph::new(Spans::from(vec![
         Span::styled(
             app.config()
@@ -538,41 +727,27 @@ fn draw_input_buffer<B: Backend>(
 }
 
 fn draw_sort_n_filter<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config
+        .default()
+        .clone()
+        .extend(panel_config.sort_and_filter().clone());
     let ui = app.config().general().sort_and_filter_ui().clone();
     let filter_by = app.explorer_config().filters();
     let sort_by = app.explorer_config().sorters();
-    let forward = Span::styled(
-        ui.sort_direction_identifiers()
-            .forward()
-            .format()
-            .to_owned()
-            .unwrap_or_default(),
-        ui.sort_direction_identifiers()
-            .forward()
-            .style()
-            .clone()
-            .into(),
-    );
-
-    let reverse = Span::styled(
-        ui.sort_direction_identifiers()
-            .reverse()
-            .format()
-            .to_owned()
-            .unwrap_or_default(),
-        ui.sort_direction_identifiers()
-            .reverse()
-            .style()
-            .clone()
-            .into(),
-    );
+    let defaultui = ui.default_identifier();
+    let forwardui = defaultui
+        .clone()
+        .extend(ui.sort_direction_identifiers().forward().clone());
+    let reverseui = defaultui
+        .clone()
+        .extend(ui.sort_direction_identifiers().reverse().clone());
 
     let mut spans = filter_by
         .iter()
@@ -580,35 +755,36 @@ fn draw_sort_n_filter<B: Backend>(
             ui.filter_identifiers()
                 .get(&f.filter())
                 .map(|u| {
+                    let ui = defaultui.clone().extend(u.clone());
                     (
                         Span::styled(
-                            u.format().to_owned().unwrap_or_default(),
-                            u.style().clone().into(),
+                            ui.format().to_owned().unwrap_or_default(),
+                            ui.style().clone().into(),
                         ),
-                        Span::raw(f.input().clone()),
+                        Span::styled(f.input().clone(), ui.style().clone().into()),
                     )
                 })
                 .unwrap_or_else(|| (Span::raw("f"), Span::raw("")))
         })
         .chain(sort_by.iter().map(|s| {
-            let direction = if s.reverse() {
-                reverse.clone()
-            } else {
-                forward.clone()
-            };
+            let direction = if s.reverse() { &reverseui } else { &forwardui };
 
             ui.sorter_identifiers()
                 .get(&s.sorter())
                 .map(|u| {
+                    let ui = defaultui.clone().extend(u.clone());
                     (
                         Span::styled(
-                            u.format().to_owned().unwrap_or_default(),
-                            u.style().clone().into(),
+                            ui.format().to_owned().unwrap_or_default(),
+                            ui.style().clone().into(),
+                        ),
+                        Span::styled(
+                            direction.format().to_owned().unwrap_or_default(),
+                            direction.style().clone().into(),
                         ),
-                        direction.clone(),
                     )
                 })
-                .unwrap_or_else(|| (Span::raw("s"), direction.clone()))
+                .unwrap_or_else(|| (Span::raw("s"), Span::raw("")))
         }))
         .zip(std::iter::repeat(Span::styled(
             ui.separator().format().to_owned().unwrap_or_default(),
@@ -628,51 +804,72 @@ fn draw_sort_n_filter<B: Backend>(
 }
 
 fn draw_logs<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config
+        .default()
+        .clone()
+        .extend(panel_config.input_and_logs().clone());
     let logs_config = app.config().general().logs().clone();
-    let logs = app
-        .logs()
-        .iter()
-        .rev()
-        .take(1)
-        .rev()
-        .map(|l| {
-            let time = l.created_at().format("%r");
-            match l.level() {
-                app::LogLevel::Info => ListItem::new(format!(
-                    "{} | {} | {}",
-                    &time,
-                    &logs_config.info().format().to_owned().unwrap_or_default(),
-                    l.message()
-                ))
-                .style(logs_config.info().style().clone().into()),
-                app::LogLevel::Success => ListItem::new(format!(
-                    "{} | {} | {}",
-                    &time,
-                    &logs_config
-                        .success()
-                        .format()
-                        .to_owned()
-                        .unwrap_or_default(),
-                    l.message()
-                ))
-                .style(logs_config.success().style().clone().into()),
-                app::LogLevel::Error => ListItem::new(format!(
-                    "{} | {} | {}",
-                    &time,
-                    &logs_config.error().format().to_owned().unwrap_or_default(),
-                    l.message()
-                ))
-                .style(logs_config.error().style().clone().into()),
-            }
-        })
-        .collect::<Vec<ListItem>>();
+    let logs = if app.logs_hidden() {
+        vec![]
+    } else {
+        app.logs()
+            .iter()
+            .rev()
+            .take(1)
+            .rev()
+            .map(|l| {
+                let time = l.created_at().format("%r");
+                match l.level() {
+                    app::LogLevel::Info => ListItem::new(format!(
+                        "{} | {} | {}",
+                        &time,
+                        &logs_config.info().format().to_owned().unwrap_or_default(),
+                        l.message()
+                    ))
+                    .style(logs_config.info().style().clone().into()),
+
+                    app::LogLevel::Warning => ListItem::new(format!(
+                        "{} | {} | {}",
+                        &time,
+                        &logs_config
+                            .warning()
+                            .format()
+                            .to_owned()
+                            .unwrap_or_default(),
+                        l.message()
+                    ))
+                    .style(logs_config.warning().style().clone().into()),
+
+                    app::LogLevel::Success => ListItem::new(format!(
+                        "{} | {} | {}",
+                        &time,
+                        &logs_config
+                            .success()
+                            .format()
+                            .to_owned()
+                            .unwrap_or_default(),
+                        l.message()
+                    ))
+                    .style(logs_config.success().style().clone().into()),
+
+                    app::LogLevel::Error => ListItem::new(format!(
+                        "{} | {} | {}",
+                        &time,
+                        &logs_config.error().format().to_owned().unwrap_or_default(),
+                        l.message()
+                    ))
+                    .style(logs_config.error().style().clone().into()),
+                }
+            })
+            .collect::<Vec<ListItem>>()
+    };
 
     let logs_list = List::new(logs).block(block(config, format!(" Logs ({}) ", app.logs().len())));
 
@@ -680,13 +877,14 @@ fn draw_logs<B: Backend>(
 }
 
 pub fn draw_nothing<B: Backend>(
-    config: BlockConfig,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
-    _app: &app::App,
-    _hb: &Handlebars,
+    app: &app::App,
+    _lua: &Lua,
 ) {
+    let panel_config = app.config().general().panel_ui();
+    let config = panel_config.default().clone();
     let nothing = Paragraph::new("").block(block(config, "".into()));
     f.render_widget(nothing, layout_size);
 }
@@ -697,21 +895,19 @@ pub fn draw_layout<B: Backend>(
     screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    hb: &Handlebars,
+    lua: &Lua,
 ) {
     match layout {
-        Layout::Nothing(config) => draw_nothing(config, f, screen_size, layout_size, app, hb),
-        Layout::Table(config) => draw_table(config, f, screen_size, layout_size, app, hb),
-        Layout::SortAndFilter(config) => {
-            draw_sort_n_filter(config, f, screen_size, layout_size, app, hb)
-        }
-        Layout::HelpMenu(config) => draw_help_menu(config, f, screen_size, layout_size, app, hb),
-        Layout::Selection(config) => draw_selection(config, f, screen_size, layout_size, app, hb),
-        Layout::InputAndLogs(config) => {
+        Layout::Nothing => draw_nothing(f, screen_size, layout_size, app, lua),
+        Layout::Table => draw_table(f, screen_size, layout_size, app, lua),
+        Layout::SortAndFilter => draw_sort_n_filter(f, screen_size, layout_size, app, lua),
+        Layout::HelpMenu => draw_help_menu(f, screen_size, layout_size, app, lua),
+        Layout::Selection => draw_selection(f, screen_size, layout_size, app, lua),
+        Layout::InputAndLogs => {
             if app.input_buffer().is_some() {
-                draw_input_buffer(config, f, screen_size, layout_size, app, hb);
+                draw_input_buffer(f, screen_size, layout_size, app, lua);
             } else {
-                draw_logs(config, f, screen_size, layout_size, app, hb);
+                draw_logs(f, screen_size, layout_size, app, lua);
             };
         }
         Layout::Horizontal { config, splits } => {
@@ -743,7 +939,7 @@ pub fn draw_layout<B: Backend>(
             splits
                 .into_iter()
                 .zip(chunks.into_iter())
-                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, hb));
+                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, lua));
         }
 
         Layout::Vertical { config, splits } => {
@@ -775,22 +971,21 @@ pub fn draw_layout<B: Backend>(
             splits
                 .into_iter()
                 .zip(chunks.into_iter())
-                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, hb));
+                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, lua));
         }
     }
 }
 
-pub fn draw<B: Backend>(f: &mut Frame<B>, app: &app::App, hb: &Handlebars) {
+pub fn draw<B: Backend>(f: &mut Frame<B>, app: &app::App, lua: &Lua) {
     let screen_size = f.size();
     let layout = app.layout().clone();
 
-    draw_layout(layout, f, screen_size, screen_size, app, hb);
+    draw_layout(layout, f, screen_size, screen_size, app, lua);
 }
 
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::config;
     use tui::style::Color;
 
     fn modifier(m: Modifier) -> Option<IndexSet<Modifier>> {
@@ -862,82 +1057,4 @@ mod test {
             }
         );
     }
-
-    #[test]
-    fn test_extend_ui_config() {
-        let a = config::UiConfig {
-            prefix: Some("a".to_string()),
-            suffix: None,
-            style: Style {
-                fg: Some(Color::Red),
-                bg: None,
-                add_modifiers: modifier(Modifier::Bold),
-                sub_modifiers: None,
-            },
-        };
-
-        let b = config::UiConfig {
-            prefix: None,
-            suffix: Some("b".to_string()),
-            style: Style {
-                fg: None,
-                bg: Some(Color::Blue),
-                add_modifiers: None,
-                sub_modifiers: modifier(Modifier::Dim),
-            },
-        };
-
-        let c = config::UiConfig {
-            prefix: Some("cp".to_string()),
-            suffix: Some("cs".to_string()),
-            style: Style {
-                fg: Some(Color::Cyan),
-                bg: Some(Color::Magenta),
-                add_modifiers: modifier(Modifier::CrossedOut),
-                sub_modifiers: modifier(Modifier::Italic),
-            },
-        };
-
-        assert_eq!(
-            a.clone().extend(b.clone()),
-            config::UiConfig {
-                prefix: Some("a".to_string()),
-                suffix: Some("b".to_string()),
-                style: Style {
-                    fg: Some(Color::Red),
-                    bg: Some(Color::Blue),
-                    add_modifiers: modifier(Modifier::Bold),
-                    sub_modifiers: modifier(Modifier::Dim),
-                },
-            }
-        );
-
-        assert_eq!(
-            b.clone().extend(a.clone()),
-            config::UiConfig {
-                prefix: Some("a".to_string()),
-                suffix: Some("b".to_string()),
-                style: Style {
-                    fg: Some(Color::Red),
-                    bg: Some(Color::Blue),
-                    add_modifiers: modifier(Modifier::Bold),
-                    sub_modifiers: modifier(Modifier::Dim),
-                },
-            }
-        );
-
-        assert_eq!(
-            a.clone().extend(c.clone()),
-            config::UiConfig {
-                prefix: Some("cp".to_string()),
-                suffix: Some("cs".to_string()),
-                style: Style {
-                    fg: Some(Color::Cyan),
-                    bg: Some(Color::Magenta),
-                    add_modifiers: modifier(Modifier::CrossedOut),
-                    sub_modifiers: modifier(Modifier::Italic),
-                },
-            }
-        );
-    }
 }
